INTEGRATION_ARTIFACT_BUCKET = "integration_artifact_s3_bucket"
ENVIRONMENT = "environment"
ARTIFACT_MANAGER = "integration_artifact_manager"
CYPRESS_TEST_DIR = "cypress_test_dir"
INTEGRATION_TARGET_URL = "integration_target_url"
TAVERN_ADDITIONAL_ARGS = "tavern_addition_args"
WORKING_TEST_DIR = "verify_environment_new_test_dir"
TAVERN_TEST_DIR = "tavern_test_dir"
DEFAULT_TAVERN_TEST_DIR = "src/integrationtest/tavern"
PROMOTE_ARTIFACT = "promote"
ROLE = "role"
APPLICATION = "application"
APPLICATION_GROUP = "application_group"
